@@ -1,5 +1,7 @@
+import copy
 import csv
 import json
+import gzip
 import pickle
 import logging
 import itertools
@@ -9,7 +11,8 @@ from adeft.modeling.classify import load_model_info
 from adeft import available_shortforms as available_adeft_models
 from .term import Term
 from .process import normalize, replace_dashes, replace_greek_uni, \
-    replace_greek_latin, replace_greek_spelled_out, depluralize
+    replace_greek_latin, replace_greek_spelled_out, depluralize, \
+    replace_roman_arabic
 from .scorer import generate_match, score, score_namespace
 from .resources import get_gilda_models, get_grounding_terms
 
@@ -83,9 +86,13 @@ class Grounder(object):
         lookups.add(greek_replaced)
         greek_replaced = normalize(replace_greek_spelled_out(raw_str))
         lookups.add(greek_replaced)
+        # We try exchanging roman and arabic numerals
+        roman_arabic = normalize(replace_roman_arabic(raw_str))
+        lookups.add(roman_arabic)
         # Finally, we attempt to depluralize the word
         depluralized = normalize(depluralize(raw_str)[0])
         lookups.add(depluralized)
+
         logger.debug('Looking up the following strings: %s' %
                      ', '.join(lookups))
         return lookups
@@ -231,6 +238,7 @@ class Grounder(object):
         for _, entry_group in entry_groups:
             entries = sorted(list(entry_group), key=lambda x: x.score,
                              reverse=True)
+            entries[0].subsumed_terms = [e.term for e in entries[1:]]
             unique_entries.append(entries[0])
         # Return the list of unique entries
         return unique_entries
@@ -290,13 +298,21 @@ class ScoredMatch(object):
         The Match object characterizing the match to the Term.
     disambiguation : Optional[dict]
         Meta-information about disambiguation, when available.
+    subsumed_terms : Optional[list[gilda.grounder.Term]]
+        A list of additional Term objects that also matched, have the same
+        db/id value as the term associated with the match, but were further
+        down the score ranking. In some cases examining the subsumed terms
+        associated with a match can provide additional metadata in
+        downstream applications.
     """
-    def __init__(self, term, score, match, disambiguation=None):
+    def __init__(self, term, score, match, disambiguation=None,
+                 subsumed_terms=None):
         self.term = term
         self.url = term.get_idenfiers_url()
         self.score = score
         self.match = match
         self.disambiguation = disambiguation
+        self.subsumed_terms = subsumed_terms if subsumed_terms else None
 
     def __str__(self):
         disamb_str = '' if self.disambiguation is None else \
@@ -316,6 +332,9 @@ class ScoredMatch(object):
         }
         if self.disambiguation is not None:
             js['disambiguation'] = self.disambiguation
+        if self.subsumed_terms:
+            js['subsumed_terms'] = [term.to_json()
+                                    for term in self.subsumed_terms]
         return js
 
     def multiply(self, value):
@@ -339,9 +358,12 @@ def load_terms_file(terms_file):
         A lookup dictionary whose keys are normalized entity texts, and values
         are lists of Terms with that normalized entity text.
     """
-    with open(terms_file, 'r', encoding='utf-8') as fh:
+    with gzip.open(terms_file, 'rt', encoding='utf-8') as fh:
         entries = {}
-        for row in csv.reader(fh, delimiter='\t'):
+        reader = csv.reader(fh, delimiter='\t')
+        # Skip header
+        next(reader)
+        for row in reader:
             row_nones = [r if r else None for r in row]
             entry = Term(*row_nones)
             if row[0] in entries:
@@ -378,9 +400,9 @@ def load_adeft_models():
     return adeft_disambiguators
 
 
-def load_gilda_models():
-    with open(get_gilda_models(), 'rb') as fh:
+def load_gilda_models(cutoff=0.7):
+    with gzip.open(get_gilda_models(), 'rb') as fh:
         models_raw = pickle.load(fh)
     models = {k: load_model_info(v['cl']) for k, v in models_raw.items()}
-    models = {k: v for k, v in models.items() if v.stats['f1']['mean'] > 0.7}
+    models = {k: v for k, v in models.items() if v.stats['f1']['mean'] > cutoff}
     return models
